local AddonName, fPB = ...
local L = fPB.L

local db = {}
local UpdateAllNameplates = fPB.UpdateAllNameplates

local 	GetSpellInfo, tonumber, pairs, table_sort, table_insert =
		GetSpellInfo, tonumber, pairs, table.sort, table.insert

function fPB.OptionsOnEnable()
	db = fPB.db.profile

	fPB.BuildSpellList()
end

fPB.tooltip = fPB.tooltip or CreateFrame("GameTooltip", AddonName.."Tooltip", UIParent, "GameTooltipTemplate")
local tooltip = fPB.tooltip
tooltip:Show()
tooltip:SetOwner(UIParent, "ANCHOR_NONE")


local minIconSize = 10
local maxIconSize = 100
local minTextSize = 6
local maxTextSize = 30
local minInterval = 0
local maxInterval = 80


fPB.MainOptionTable = {
	name = L["Display options"],
	type = "group",
	childGroups = "tab",
	get = function(info)
        return db[info[#info]]
    end,

	set = function(info, value)
        db[info[#info]] = value
		UpdateAllNameplates()
    end,

	args = {
		displayConditions = {
			order = 1,
			name = L["Display conditions"],
			type = "group",
			args = {
				showDebuffs = {
					order = 1,
					type = "select",
					style = "dropdown",
					name = L["Show debuffs"],
					values = {
							L["All"],
							L["Mine + SpellList"],
							L["Only SpellList"],
							L["Only mine"],
					},
				},
				showBuffs = {
					order = 2,
					type = "select",
					style = "dropdown",
					name = L["Show buffs"],
					values = {
							L["All"],
							L["Mine + SpellList"],
							L["Only SpellList"],
							L["Only mine"],
					},
				},
				hidePermanent = {
					order = 3,
					type = "toggle",
					name = L["Hide permanent effects"],
					desc = L["Do not show effects without duration."],
				},
				showOnlyInCombat = {
					order = 6,
					type = "toggle",
					name = L["Player in combat"],
					desc = L["Show only if player is in combat"],
					set = function(info,value)
						db.showOnlyInCombat = value
						UpdateAllNameplates()
						if value then
							fPB.RegisterCombat()
							if not InCombatLockdown() then fPB.Events:UnregisterEvent("UNIT_AURA") end
						else
							fPB.UnregisterCombat()
							fPB.Events:RegisterEvent("UNIT_AURA")
						end
					end,
				},
				showUnitInCombat = {
					order = 7,
					type = "toggle",
					name = L["Unit in combat"],
					desc = L["Show only if unit is in combat"],
				},
				break1 = {
					order = 8,
					type = "header",
					name = "",
				},
				showOnPlayers = {
					order = 9,
					type = "toggle",
					name = L["Players"],
					desc = L["Show on players"],
				},
				blank1 = {
					order = 10,
					type = "description",
					name = "",
					width = "normal",
				},
				showOnEnemy = {
					order = 11,
					type = "toggle",
					name = L["Enemies"],
					desc = L["Show on enemies"],
				},
				showOnPets = {
					order = 12,
					type = "toggle",
					name = L["Pets"],
					desc = L["Show on pets"],
				},
				blank2 = {
					order = 13,
					type = "description",
					name = "",
					width = "normal",
				},
				showOnFriend = {
					order = 14,
					type = "toggle",
					name = L["Allies"],
					desc = L["Show on allies"],
				},
				showOnNPC = {
					order = 15,
					type = "toggle",
					name = L["NPCs"],
					desc = L["Show on NPCs"],
				},
				blank3 = {
					order = 16,
					type = "description",
					name = "",
					width = "normal",
				},
				showOnNeutral = {
					order = 17,
					type = "toggle",
					name = L["Neutrals"],
					desc = L["Show on neutral characters"],
				},
			},
		},
		styleSettings = {
			order = 2,
			name = L["Style settings"],
			type = "group",
			args = {
				iconsSize = {
					order = 1,
					type = "header",
					name = L["Icons Size"],
				},
				baseWidth = {
					order = 2,
					type = "range",
					name = L["Base width"],
					min = minIconSize,
					max = maxIconSize,
					step = 1,
				},
				baseHeight = {
					order = 3,
					type = "range",
					name = L["Base height"],
					min = minIconSize,
					max = maxIconSize,
					step = 1,
				},
				myScale = {
					order = 4,
					type = "range",
					name = L["Larger self spells"],
					desc = L["Show self spells x% bigger."],
					min = 0,
					max = 1,
					step = 0.1,
					isPercent = true,
				},
				headerStack = {
					order = 5,
					type = "header",
					name = L["Stacks & Duration"],
				},
				font = {
					order = 6,
					type = "select",
					name = L["Font"],
					values = fPB.LSM:HashTable("font"),
					dialogControl = "LSM30_Font",
					set = function(info, value)
						db.font = value
						fPB.font = fPB.LSM:Fetch("font", db.font)
						UpdateAllNameplates()
					end,
				},
				stackSize = {
					order = 7,
					type = "range",
					name = L["Stack font size"],
					min = minTextSize,
					max = maxTextSize,
					step = 1,
				},
				durationSize = {
					order = 8,
					type = "range",
					name = L["Duration font size"],
					min = minTextSize,
					max = maxTextSize,
					step = 1,
				},
				showDuration = {
					order = 9,
					type = "toggle",
					name = L["Duration under icon"],
					desc = L["Show remaining duration under icon"],
				},
				showStdCooldown = {
					order = 10,
					type = "toggle",
					name = L["Duration on icon"],
					desc = L["Support standart blizzard or OmniCC"],
				},
				showStdSwipe = {
					order = 11,
					type = "toggle",
					name = L["Show 'clock' animation"],
					desc = L["Also will show duration if OmniCC installed, regardless of the previous option"],
				},
				blinkTimeleft = {
					order	= 11.5,
					name = L["Blink when close to expiring"],
					desc = L["Blink spell if below x% time left (only if it's below 60 seconds)"],
					type = "range",
					min		= 0,
					max		= 0.5,
					step	= 0.05,
					isPercent = true,
				},
				headerBorder = {
					order = 12,
					type = "header",
					name = L["Border"],
				},
				borderStyle = {
					order = 13,
					type = "select",
					style = "dropdown",
					name = L["Border Style"],
					values = {
						L["Square"],
						"Blizzard",
						L["None"],
					},
				},
				colorizeBorder = {
					order = 14,
					type = "toggle",
					name = L["Color debuff border by type"],
					desc = L["If not checked - physical used for all debuff types"],
					width = "double",
					disabled = function() return db.borderStyle == 3 end,
				},

				colorsPhysical = {
					order = 15,
					type = "color",
					name = L["Physical"],
					get = function(info)
						return db.colorTypes.none[1], db.colorTypes.none[2], db.colorTypes.none[3], 1
					end,
					set = function(info, r, g, b)
						db.colorTypes.none = {r, g, b}
					end,
				},
				colorsMagic = {
					order = 16,
					type = "color",
					name = L["Magic"],
					get = function(info)
						return db.colorTypes.Magic[1], db.colorTypes.Magic[2], db.colorTypes.Magic[3], 1
					end,
					set = function(info, r, g, b)
						db.colorTypes.Magic = {r, g, b}
					end,
				},
				colorsCurse = {
					order = 17,
					type = "color",
					name = L["Curse"],
					get = function(info)
						return db.colorTypes.Curse[1], db.colorTypes.Curse[2], db.colorTypes.Curse[3], 1
					end,
					set = function(info, r, g, b)
						db.colorTypes.Curse = {r, g, b}
					end,
				},
				colorsDisease = {
					order = 18,
					type = "color",
					name = L["Disease"],
					get = function(info)
						return db.colorTypes.Disease[1], db.colorTypes.Disease[2], db.colorTypes.Disease[3], 1
					end,
					set = function(info, r, g, b)
						db.colorTypes.Disease = {r, g, b}
					end,
				},
				colorsPoison = {
					order = 19,
					type = "color",
					name = L["Poison"],
					get = function(info)
						return db.colorTypes.Poison[1], db.colorTypes.Poison[2], db.colorTypes.Poison[3], 1
					end,
					set = function(info, r, g, b)
						db.colorTypes.Poison = {r, g, b}
					end,
				},
				colorsBuff = {
					order = 20,
					type = "color",
					name = L["Buffs"],
					get = function(info)
						return db.colorTypes.Buff[1], db.colorTypes.Buff[2], db.colorTypes.Buff[3], 1
					end,
					set = function(info, r, g, b)
						db.colorTypes.Buff = {r, g, b}
					end,
				},
			},

		},
		positionSettings = {
			order = 3,
			name = L["Position Settings"],
			type = "group",
			args = {
				xInterval = {
					order = 1,
					type = "range",
					name = L["Interval X"],
					desc = L["Horizontal spacing between icons"],
					min = minInterval,
					max = maxInterval,
					step = 1,
				},
				yInterval = {
					order = 2,
					type = "range",
					name = L["Interval Y"],
					desc = L["Vertical spacing between icons"],
					min = minInterval,
					max = maxInterval,
					step = 1,
				},
				blank1 = {
					order = 3,
					type = "description",
					name = "",
					width = "normal",
				},
				buffPerLine = {
					order = 4,
					type = "range",
					name = L["Icons per row"],
					desc = L["If more icons they will be moved to a new row"],
					min = 1,
					max = 20,
					step = 1,
				},
				numLines = {
					order = 5,
					type = "range",
					name = L["Max rows"],
					desc = L["Excess buffs will not be displayed"],
					min = 1,
					max = 10,
					step = 1,
				},
				blank2 = {
					order = 6,
					type = "description",
					name = "",
					width = "normal",
				},
				xOffset = {
					order = 7,
					type = "range",
					name = L["Offset X"],
					desc = L["Horizontal offset of buff frame"],
					min = -256,
					max = 256,
					step = 1,
				},
				yOffset = {
					order = 8,
					type = "range",
					name = L["Offset Y"],
					desc = L["Vertical offset of buff frame"],
					min = -256,
					max = 256,
					step = 1,
				},
				break1 = {
					order = 9,
					type = "header",
					name = "",
				},
				parentWorldFrame = {
					order = 10,
					type = "toggle",
					name = L["Always show icons with full opacity and size"],
					desc = L["Icons will not change on nontargeted nameplates.\n\n|cFFFF0000REALLY NOT RECOMMEND|r\nWhen icons overlay there will be mess of textures, digits etc."],
					width = "full",
				},
			},
		},
	},
}

local newSpellName
local newSpellID

fPB.SpellsTable = {
	name = L["Specific spells"],
	type = "group",
	childGroups = "tree",
	args = {
		addSpell = {
			order = 1,
			type = "input",
			name = L["Add new spell to list"],
			desc = L["Enter spell ID or name (case sensitive)\nand press OK"],
			set = function(info, value)
				if value then
					local spellID = tonumber(value)
					if spellID then
						local spellName = GetSpellInfo(spellID)
						if spellName then
							newSpellName = spellName
							newSpellID = spellID
						end
						fPB.AddNewSpell(newSpellName, newSpellID)
					else
						newSpellName = value
						fPB.AddNewSpell(newSpellName)
					end
				end
			end,
			get = function(info)
				return newSpellName
			end,
		},
		blank = {
			order = 2,
			type = "description",
			name = "",
			width = "normal",
		},
		showSpellID = {
			type = "toggle",
			order = 3,
			name = L["Show spell ID in tooltips"],
			desc = L["Usefull for configuring spell list.\nRequires ReloadUI to turn off."],
			confirm = function () if db.showSpellID then return L["ReloadUI"] end end,
			get = function(info)
				return db[info[#info]]
			end,
			set = function(info,value)
				db.showSpellID = value
				if value then
					fPB.ShowSpellID()
				else
					ReloadUI()
				end
			end,
		},

		-- fills up with BuildSpellList()
	},
}

local color
local iconTexture
local TextureStringCache = {}
local description
local function TextureString(spellID)
	if not tonumber(spellID) then
		return "\124TInterface\\Icons\\Inv_misc_questionmark:0\124t"
	elseif TextureStringCache[spellID] then
		return TextureStringCache[spellID]
	else
		_,_,iconTexture = GetSpellInfo(spellID)
		if iconTexture then
			iconTexture = "\124T"..iconTexture..":0\124t"
			TextureStringCache[spellID] = iconTexture
			return iconTexture
		else
			return "\124TInterface\\Icons\\Inv_misc_questionmark:0\124t"
		end
	end
end

function fPB.BuildSpellList()
	local spellTable = fPB.SpellsTable.args
	for item in pairs(spellTable) do
		if item ~= "addSpell" and item ~= "blank" and item ~= "showSpellID" then
			spellTable[item] = nil
		end
	end
	local Spells = db.Spells
	local Ignored = db.ignoredDefaultSpells
	local spellList = {}
	for name in pairs(Spells) do
		if not Ignored[name] then
			table_insert(spellList, name)
		end
	end
	table_sort(spellList)

	for i = 1, #spellList do
		local name = spellList[i]
		local Spell = Spells[name]
		local SpellID = Spells[name].spellID
		if Spell.show == 1 then
			color = "|cFF00FF00" --green
		elseif Spell.show ==3 then
			color = "|cFFFF0000" --red
		else
			color = "|cFFFFFF00" --yellow
		end

		iconTexture = TextureString(SpellID)

		if tonumber(SpellID) then
			tooltip:SetHyperlink("spell:"..SpellID)
			spellTexture = select(3, GetSpellInfo(SpellID))
			local lines = tooltip:NumLines()
			if lines > 0 then
				spellDesc = _G[AddonName.."TooltipTextLeft"..lines] and _G[AddonName.."TooltipTextLeft"..lines]:GetText() or "??"
			end
		else
			spellDesc = L["No spell ID"]
		end

		local buildName = iconTexture.." "..color..name.." (x"..Spell.scale..")"
		if tonumber(SpellID) then
			buildName = buildName.."  id:"..SpellID.."|r"
		else
			buildName = buildName.."|r"
		end

		spellTable[name] = {
			name = buildName,
			desc = spellDesc,
			type = "group",
			order = 10 + i,
			get = function(info)
				if Spells[info[1]] then
					return Spells[info[1]][info[2]]
				end
			end,
			set = function(info, value)
				Spells[info[1]][info[2]] = value
				UpdateAllNameplates()
				fPB.BuildSpellList()
			end,
			args = {
				show = {
					order = 1,
					name = L["Show"],
					type = "select",
					style = "dropdown",
					values = {
						L["Always"],
						L["Only mine"],
						L["Never"],
					},
				},
				scale = {
					order = 2,
					name = L["Icon scale"],
					type = "range",
					min = 1,
					max = 5,
					step = 0.1,
				},
				stackSize = {
					order = 3,
					name = L["Stack font size"],
					type = "range",
					min = minTextSize,
					max = maxTextSize,
					step = 1,
				},
				durationSize = {
					order = 4,
					name = L["Duration font size"],
					type = "range",
					min = minTextSize,
					max = maxTextSize,
					step = 1,
				},
				spellID = {
					order = 5,
					type = "input",
					name = L["Spell ID"],
					get = function(info)
						if Spells[info[1]] then
							return tostring(Spells[info[1]].spellID or L["No spell ID"])
						end
					end,
					set = function(info, value)
						if value then
							local spellID = tonumber(value)
							if spellID then
								local spellName = GetSpellInfo(spellID)
								if spellName == info[1] then
									Spells[info[1]].spellID = value
								else
									Spells[info[1]].spellID = L["Incorrect ID or name"]
								end
							else
								Spells[info[1]].spellID = L["Incorrect ID or name"]
							end
						fPB.BuildSpellList()
						end
					end,
				},
				checkID = {
					order = 6,
					type = "toggle",
					name = L["Check spell ID"],
				},
				removeSpell = {
					order = 7,
					type = "execute",
					name = L["Remove spell"],
					confirm = true,
					func = function(info)
						fPB.RemoveSpell(info[1])
					end,
				},
			},
		}
	end
end

